@@ -22,11 +22,11 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.hadoop.conf.Configuration;
-import org.apache.hadoop.security.Credentials;
 import org.apache.hadoop.security.UserGroupInformation;
+import org.apache.hadoop.yarn.api.records.ContainerId;
 import org.apache.hadoop.yarn.api.records.LocalResource;
 import org.apache.hadoop.yarn.event.EventHandler;
+import org.apache.tez.common.counters.DAGCounter;
 import org.apache.tez.common.counters.TezCounters;
 import org.apache.tez.dag.api.TezException;
 import org.apache.tez.dag.api.client.DAGStatusBuilder;
@@ -38,6 +38,8 @@ import org.apache.tez.dag.records.TezDAGID;
 import org.apache.tez.dag.records.TezVertexID;
 import org.apache.tez.serviceplugins.api.DagInfo;
 
+import javax.annotation.Nullable;
+
 /**
  * Main interface to interact with the job.
  */
@@ -74,8 +76,6 @@ public interface DAG extends DagInfo {
   boolean isUber();
   String getUserName();
 
-  Configuration getConf();
-
   DAGPlan getJobPlan();
   DAGStatusBuilder getDAGStatus(Set<StatusGetOpts> statusOptions);
   DAGStatusBuilder getDAGStatus(Set<StatusGetOpts> statusOptions, long timeout)
@@ -97,4 +97,25 @@ public interface DAG extends DagInfo {
 
   org.apache.tez.dag.api.Vertex.VertexExecutionContext getDefaultExecutionContext();
 
+  /**
+   *
+   * @return the DAGScheduler that will schedule
+   * this DAG, null if it doesn't exist
+   */
+  @Nullable DAGScheduler getDAGScheduler();
+
+  void incrementDagCounter(DAGCounter counter, int incrValue);
+  void setDagCounter(DAGCounter counter, int setValue);
+  void addUsedContainer(ContainerId containerId);
+
+  /**
+   * Called by the DAGAppMaster when the DAG is started normally or in the event of recovery.
+   */
+  void onStart();
+
+  /**
+   * Called by the DAGAppMaster when the DAG is finished, or there is a currentDAG on AM stop.
+   * The implementation of this method should be idempontent.
+   */
+  void onFinish();
 }
