@@ -254,7 +254,8 @@ public class YarnTaskSchedulerService extends TaskScheduler
 
   @Override
   public Resource getAvailableResources() {
-    return amRmClient.getAvailableResources();
+    Resource resource = amRmClient.getAvailableResources();
+    return resource == null ? Resource.newInstance(0, 0) : resource;
   }
 
   @Override
@@ -490,6 +491,8 @@ public class YarnTaskSchedulerService extends TaskScheduler
 
   @Override
   public void onContainersAllocated(List<Container> containers) {
+    super.onContainersAllocated(containers);
+
     if (isStopStarted.get()) {
       LOG.info("Ignoring container allocations because application is shutting down. Num " + 
           containers.size());
@@ -1166,7 +1169,7 @@ public class YarnTaskSchedulerService extends TaskScheduler
     ContainerId[] preemptedContainers = null;
     int numPendingRequestsToService = 0;
     synchronized (this) {
-      Resource freeResources = amRmClient.getAvailableResources();
+      Resource freeResources = this.getAvailableResources();
       if (LOG.isDebugEnabled()) {
         LOG.debug(constructPreemptionPeriodicLog(freeResources));
       } else {
@@ -1606,6 +1609,8 @@ public class YarnTaskSchedulerService extends TaskScheduler
         heldContainers.put(container.getId(),
             new HeldContainer(container, heldContainer.getNextScheduleTime(),
                 heldContainer.getContainerExpiryTime(), assigned, this.containerSignatureMatcher));
+      } else { // if a held container is not new, it's most probably reused
+        getContext().containerReused(container);
       }
       heldContainer.setLastTaskInfo(assigned);
     }
@@ -2399,4 +2404,9 @@ public class YarnTaskSchedulerService extends TaskScheduler
             : "null");
     }
   }
+
+  @Override
+  public int getHeldContainersCount() {
+    return heldContainers.size();
+  }
 }
