@@ -304,7 +304,7 @@ public class AMNodeImpl implements AMNode {
     @Override
     public AMNodeState transition(AMNodeImpl node, AMNodeEvent nEvent) {
       AMNodeEventTaskAttemptEnded event = (AMNodeEventTaskAttemptEnded) nEvent;
-      LOG.info("Attempt " + (event.failed() ? "failed" : "killed") + "on node: " + node.getNodeId()
+      LOG.info("Attempt " + (event.failed() ? "failed" : "killed") + " on node: " + node.getNodeId()
           + " TA: " + event.getTaskAttemptId()
           + ", container: " + event.getContainerId() + ", numFailedTAs: "
           + node.numFailedTAs);
@@ -509,4 +509,10 @@ public class AMNodeImpl implements AMNode {
       this.writeLock.unlock();
     }
   }
+
+  public String toString() {
+    return String.format(
+        "{AMNodeImpl: nodeId: %s, state: %s, containers: %d, completed containers: %d, healthy: %s, blackListed: %s}",
+        nodeId, getState(), getContainers().size(), completedContainers.size(), !isUnhealthy(), isBlacklisted());
+  }
 }
