@@ -25,6 +25,8 @@ import static org.mockito.Mockito.doReturn;
 import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.verify;
 import static org.mockito.Mockito.when;
 
 import java.io.IOException;
@@ -42,18 +44,22 @@ import java.util.concurrent.locks.ReentrantLock;
 
 import org.apache.commons.lang.StringUtils;
 import org.apache.tez.common.DrainDispatcher;
+import org.apache.tez.common.counters.DAGCounter;
 import org.apache.tez.common.counters.Limits;
 import org.apache.tez.common.counters.TezCounters;
 import org.apache.tez.dag.api.TezConstants;
 import org.apache.tez.dag.app.dag.event.DAGEventTerminateDag;
 import org.apache.tez.hadoop.shim.DefaultHadoopShim;
 import org.apache.tez.hadoop.shim.HadoopShim;
+import org.junit.Rule;
+import org.junit.rules.TestName;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.security.Credentials;
 import org.apache.hadoop.yarn.api.records.ApplicationAttemptId;
 import org.apache.hadoop.yarn.api.records.ApplicationId;
+import org.apache.hadoop.yarn.api.records.ContainerId;
 import org.apache.hadoop.yarn.api.records.Resource;
 import org.apache.hadoop.yarn.event.EventHandler;
 import org.apache.hadoop.yarn.util.Clock;
@@ -161,6 +167,9 @@ import com.google.protobuf.ByteString;
 
 public class TestDAGImpl {
 
+  @Rule
+  public TestName testName = new TestName();
+
   private static final Logger LOG = LoggerFactory.getLogger(TestDAGImpl.class);
   private DAGPlan dagPlan;
   private TezDAGID dagId;
@@ -426,7 +435,7 @@ public class TestDAGImpl {
   }
 
   // Create a plan with 3 vertices: A, B, C. Group(A,B)->C
-  static DAGPlan createGroupDAGPlan() {
+  static DAGPlan createGroupDAGPlan(String dagName) {
     LOG.info("Setting up group dag plan");
     int dummyTaskCount = 1;
     Resource dummyTaskResource = Resource.newInstance(1, 1);
@@ -440,7 +449,7 @@ public class TestDAGImpl {
         ProcessorDescriptor.create("Processor"),
         dummyTaskCount, dummyTaskResource);
 
-    DAG dag = DAG.create("testDag");
+    DAG dag = DAG.create("DAG-" + dagName);
     String groupName1 = "uv12";
     OutputCommitterDescriptor ocd = OutputCommitterDescriptor.create(
         TotalCountingOutputCommitter.class.getName());
@@ -906,7 +915,7 @@ public class TestDAGImpl {
     doReturn(defaultShim).when(groupAppContext).getHadoopShim();
 
     groupDagId = TezDAGID.getInstance(appAttemptId.getApplicationId(), 3);
-    groupDagPlan = createGroupDAGPlan();
+    groupDagPlan = createGroupDAGPlan(testName.getMethodName());
     groupDag = new DAGImpl(groupDagId, conf, groupDagPlan,
         dispatcher.getEventHandler(), taskCommunicatorManagerInterface,
         fsTokens, clock, "user", thh,
@@ -2349,4 +2358,22 @@ public class TestDAGImpl {
 
   }
 
+  @SuppressWarnings("unchecked")
+  @Test(timeout = 5000)
+  public void testTotalContainersUsedCounter() {
+    initDAG(mrrDag);
+    dispatcher.await();
+    startDAG(mrrDag);
+    dispatcher.await();
+
+    DAGImpl spy = spy(mrrDag);
+    spy.addUsedContainer(mock(ContainerId.class));
+    spy.addUsedContainer(mock(ContainerId.class));
+
+    spy.onFinish();
+    // 2 calls to addUsedContainer, obviously, we did it here
+    verify(spy, times(2)).addUsedContainer(any(ContainerId.class));
+    // 1 call to setDagCounter, which happened at dag.onFinish
+    verify(spy).setDagCounter(DAGCounter.TOTAL_CONTAINERS_USED, 2);
+  }
 }
