@@ -37,7 +37,9 @@ import org.apache.tez.test.service.rpc.TezTestServiceProtocolProtos.IOSpecProto;
 import org.apache.tez.test.service.rpc.TezTestServiceProtocolProtos.TaskSpecProto;
 import org.apache.tez.test.service.rpc.TezTestServiceProtocolProtos.TaskSpecProto.Builder;
 
-public class ProtoConverters {
+public final class ProtoConverters {
+
+  private ProtoConverters() {}
 
   public static TaskSpec getTaskSpecfromProto(TaskSpecProto taskSpecProto) {
     TezTaskAttemptID taskAttemptID =
