@@ -28,6 +28,7 @@ public interface FetcherCallback {
       FetchedInput fetchedInput, long fetchedBytes, long decompressedLength, long copyDuration)
       throws IOException;
   
-  public void fetchFailed(String host, InputAttemptIdentifier srcAttemptIdentifier, boolean connectFailed);
+  public void fetchFailed(String host, InputAttemptFetchFailure srcAttemptFetchFailure,
+      boolean connectFailed);
 
 }
