@@ -396,13 +396,13 @@ class ShuffleScheduler {
 
     this.maxFailedUniqueFetches = Math.min(numberOfInputs, 5);
     referee.start();
-    this.maxFetchFailuresBeforeReporting = 
+    this.maxFetchFailuresBeforeReporting =
         conf.getInt(
             TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_FETCH_FAILURES_LIMIT,
             TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_FETCH_FAILURES_LIMIT_DEFAULT);
-    this.reportReadErrorImmediately = 
+    this.reportReadErrorImmediately =
         conf.getBoolean(
-            TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_NOTIFY_READERROR, 
+            TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_NOTIFY_READERROR,
             TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_NOTIFY_READERROR_DEFAULT);
     this.verifyDiskChecksum = conf.getBoolean(
         TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_FETCH_VERIFY_DISK_CHECKSUM,
@@ -415,7 +415,7 @@ class ShuffleScheduler {
     this.maxTaskOutputAtOnce = Math.max(1, Math.min(75, conf.getInt(
             TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_FETCH_MAX_TASK_OUTPUT_AT_ONCE,
             TezRuntimeConfiguration.TEZ_RUNTIME_SHUFFLE_FETCH_MAX_TASK_OUTPUT_AT_ONCE_DEFAULT)));
-    
+
     this.skippedInputCounter = inputContext.getCounters().findCounter(TaskCounter.NUM_SKIPPED_INPUTS);
     this.firstEventReceived = inputContext.getCounters().findCounter(TaskCounter.FIRST_EVENT_RECEIVED);
     this.lastEventReceived = inputContext.getCounters().findCounter(TaskCounter.LAST_EVENT_RECEIVED);
@@ -689,7 +689,7 @@ class ShuffleScheduler {
       LOG.warn("Duplicate fetch of input no longer needs to be fetched: "
           + srcAttemptIdentifier);
       // free the resource - specially memory
-      
+
       // If the src does not generate data, output will be null.
       if (output != null) {
         output.abort();
@@ -759,20 +759,22 @@ class ShuffleScheduler {
     }
   }
 
-  public synchronized void copyFailed(InputAttemptFetchFailure fetchFailure, MapHost host,
+  public void copyFailed(InputAttemptFetchFailure fetchFailure, MapHost host,
       boolean readError, boolean connectError) {
     failedShuffleCounter.increment(1);
     inputContext.notifyProgress();
-    int failures = incrementAndGetFailureAttempt(fetchFailure.getInputAttemptIdentifier());
+    int failures;
 
-    if (!fetchFailure.isLocalFetch()) {
-      /**
-       * Track the number of failures that has happened since last completion.
-       * This gets reset on a successful copy.
-       */
-      failedShufflesSinceLastCompletion++;
+    synchronized (this) {
+      failures = incrementAndGetFailureAttempt(fetchFailure.getInputAttemptIdentifier());
+      if (!fetchFailure.isLocalFetch()) {
+        /**
+         * Track the number of failures that has happened since last completion.
+         * This gets reset on a successful copy.
+         */
+        failedShufflesSinceLastCompletion++;
+      }
     }
-
     /**
      * Inform AM:
      *    - In case of read/connect error
@@ -794,14 +796,18 @@ class ShuffleScheduler {
     }
 
     //Restart consumer in case shuffle is not healthy
-    if (!isShuffleHealthy(fetchFailure)) {
+    try {
+      checkShuffleHealthy(fetchFailure);
+    } catch (IOException e) {
+      // reportException should be called outside synchronized(this) due to TEZ-4334
+      exceptionReporter.reportException(e);
       return;
     }
 
     penalizeHost(host, failures);
   }
 
-  private boolean isAbortLimitExceeedFor(InputAttemptIdentifier srcAttempt) {
+  private void isAbortLimitExceeedFor(InputAttemptIdentifier srcAttempt) throws IOException {
     int attemptFailures = getFailureCount(srcAttempt);
     if (attemptFailures >= abortFailureLimit) {
       // This task has seen too many fetch failures - report it as failed. The
@@ -816,15 +822,11 @@ class ShuffleScheduler {
           inputContext.getSourceVertexName(),
           srcAttempt.getInputIdentifier(),
           srcAttempt.getAttemptNumber()) + ". threshold=" + abortFailureLimit;
-      IOException ioe = new IOException(errorMsg);
-      // Shuffle knows how to deal with failures post shutdown via the onFailure hook
-      exceptionReporter.reportException(ioe);
-      return true;
+      throw new IOException(errorMsg);
     }
-    return false;
   }
 
-  private void penalizeHost(MapHost host, int failures) {
+  private synchronized void penalizeHost(MapHost host, int failures) {
     host.penalize();
 
     HostPort hostPort = new HostPort(host.getHost(), host.getPort());
@@ -1008,37 +1010,38 @@ class ShuffleScheduler {
     return fetcherHealthy;
   }
 
-  boolean isShuffleHealthy(InputAttemptFetchFailure fetchFailure) {
+  /**
+   * This method checks if the current shuffle is healthy and throw IOException if it's not,
+   * then the caller is supposed to handle the IOException.
+   */
+  private synchronized void checkShuffleHealthy(InputAttemptFetchFailure fetchFailure)
+      throws IOException {
     InputAttemptIdentifier srcAttempt = fetchFailure.getInputAttemptIdentifier();
-    if (isAbortLimitExceeedFor(srcAttempt)) {
-      return false;
-    }
-
-    final float MIN_REQUIRED_PROGRESS_PERCENT = minReqProgressFraction;
-    final float MAX_ALLOWED_STALL_TIME_PERCENT = maxStallTimeFraction;
+    // supposed to throw IOException if exceeded
+    isAbortLimitExceeedFor(srcAttempt);
 
     int doneMaps = numInputs - remainingMaps.get();
 
     String logContext = "srcAttempt=" + srcAttempt.toString();
     boolean fetcherHealthy = isFetcherHealthy(logContext);
-    
+
     // check if the reducer has progressed enough
     boolean reducerProgressedEnough =
       (((float)doneMaps / numInputs)
-          >= MIN_REQUIRED_PROGRESS_PERCENT);
+          >= minReqProgressFraction);
 
     // check if the reducer is stalled for a long time
     // duration for which the reducer is stalled
     int stallDuration =
       (int)(System.currentTimeMillis() - lastProgressTime);
-    
+
     // duration for which the reducer ran with progress
     int shuffleProgressDuration =
       (int)(lastProgressTime - startTime);
 
     boolean reducerStalled = (shuffleProgressDuration > 0) &&
       (((float)stallDuration / shuffleProgressDuration)
-          >= MAX_ALLOWED_STALL_TIME_PERCENT);
+          >= maxStallTimeFraction);
 
     // kill if not healthy and has insufficient progress
     if ((failureCounts.size() >= maxFailedUniqueFetches ||
@@ -1059,10 +1062,8 @@ class ShuffleScheduler {
         LOG.debug("Host failures=" + hostFailures.keySet());
       }
       // Shuffle knows how to deal with failures post shutdown via the onFailure hook
-      exceptionReporter.reportException(new IOException(errorMsg, fetchFailure.getCause()));
-      return false;
+      throw new IOException(errorMsg, fetchFailure.getCause());
     }
-    return true;
   }
 
   public synchronized void addKnownMapOutput(String inputHostName,
@@ -1095,7 +1096,7 @@ class ShuffleScheduler {
       notifyAll();
     }
   }
-  
+
   public void obsoleteInput(InputAttemptIdentifier srcAttempt) {
     // The incoming srcAttempt does not contain a path component.
     LOG.info(srcNameTrimmed + ": " + "Adding obsolete input: " + srcAttempt);
@@ -1122,7 +1123,7 @@ class ShuffleScheduler {
       obsoleteInputs.add(srcAttempt);
     }
   }
-  
+
   public synchronized void putBackKnownMapOutput(MapHost host,
                                                  InputAttemptIdentifier srcAttempt) {
     host.addKnownMap(srcAttempt);
@@ -1300,7 +1301,7 @@ class ShuffleScheduler {
   static class Penalty implements Delayed {
     MapHost host;
     private long endTime;
-    
+
     Penalty(MapHost host, long delay) {
       this.host = host;
       this.endTime = System.currentTimeMillis() + delay;
@@ -1315,7 +1316,7 @@ class ShuffleScheduler {
       long other = ((Penalty) o).endTime;
       return endTime == other ? 0 : (endTime < other ? -1 : 1);
     }
-    
+
   }
 
   /**
@@ -1349,7 +1350,7 @@ class ShuffleScheduler {
       }
     }
   }
-  
+
 
   void setInputFinished(int inputIndex) {
     synchronized(finishedMaps) {
@@ -1395,7 +1396,7 @@ class ShuffleScheduler {
         }
 
         if (LOG.isDebugEnabled()) {
-          LOG.debug(srcNameTrimmed + ": " + "NumCompletedInputs: {}" + (numInputs - remainingMaps.get()));
+          LOG.debug("{}: NumCompletedInputs: {}", srcNameTrimmed, (numInputs - remainingMaps.get()));
         }
 
         // Ensure there's memory available before scheduling the next Fetcher.
