@@ -41,7 +41,7 @@ import org.apache.tez.runtime.library.output.OrderedPartitionedKVOutput;
 import org.apache.tez.runtime.library.output.UnorderedPartitionedKVOutput;
 
 import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Preconditions;
+import org.apache.tez.common.Preconditions;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Maps;
 
@@ -244,9 +244,7 @@ public class WeightedScalingMemoryDistributor implements InitialMemoryAllocator
       requestType = RequestType.PARTITIONED_UNSORTED_OUTPUT;
     } else {
       requestType = RequestType.OTHER;
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Falling back to RequestType.OTHER for class: " + className);
-      }
+      LOG.debug("Falling back to RequestType.OTHER for class: {}", className);
     }
     return requestType;
   }
