@@ -37,11 +37,11 @@ import java.util.List;
 import java.util.concurrent.ExecutorService;
 
 import static org.junit.Assert.assertTrue;
-import static org.mockito.Matchers.any;
-import static org.mockito.Matchers.anyInt;
-import static org.mockito.Matchers.anyString;
-import static org.mockito.Matchers.eq;
+import static org.mockito.Mockito.any;
+import static org.mockito.Mockito.anyInt;
+import static org.mockito.Mockito.anyString;
 import static org.mockito.Mockito.doReturn;
+import static org.mockito.Mockito.eq;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.spy;
 import static org.mockito.Mockito.times;
@@ -95,8 +95,8 @@ public class TestShuffleInputEventHandlerOrderedGrouped {
           @Override
           public ExecutorService answer(InvocationOnMock invocation) throws Throwable {
             return sharedExecutor.createExecutorService(
-                invocation.getArgumentAt(0, Integer.class),
-                invocation.getArgumentAt(1, String.class));
+                invocation.getArgument(0, Integer.class),
+                invocation.getArgument(1, String.class));
           }
         });
     return inputContext;
@@ -272,7 +272,7 @@ public class TestShuffleInputEventHandlerOrderedGrouped {
     handler.handleEvents(Collections.singletonList(dme2));
 
     // task should issue kill request
-    verify(scheduler, times(1)).killSelf(any(IOException.class), any(String.class));
+    verify(scheduler, times(1)).killSelf(any(), any());
   }
 
   @Test (timeout = 5000)
@@ -307,7 +307,7 @@ public class TestShuffleInputEventHandlerOrderedGrouped {
     handler.handleEvents(events);
 
     // task should issue kill request, as inputs are scheduled for download already.
-    verify(scheduler, times(1)).killSelf(any(IOException.class), any(String.class));
+    verify(scheduler, times(1)).killSelf(any(), any());
   }
 
   @Test(timeout = 5000)
@@ -348,8 +348,8 @@ public class TestShuffleInputEventHandlerOrderedGrouped {
     events.add(dme);
     handler.handleEvents(events);
     InputAttemptIdentifier expectedIdentifier = new InputAttemptIdentifier(targetIdx, 0);
-    verify(scheduler).copySucceeded(eq(expectedIdentifier), any(MapHost.class), eq(0l),
-        eq(0l), eq(0l), any(MapOutput.class), eq(true));
+    verify(scheduler).copySucceeded(eq(expectedIdentifier), any(), eq(0L),
+            eq(0L), eq(0L), any(), eq(true));
   }
 
   @Test(timeout = 5000)
@@ -362,8 +362,8 @@ public class TestShuffleInputEventHandlerOrderedGrouped {
     events.add(dme);
     handler.handleEvents(events);
     InputAttemptIdentifier expectedIdentifier = new InputAttemptIdentifier(targetIdx, 0);
-    verify(scheduler).copySucceeded(eq(expectedIdentifier), any(MapHost.class), eq(0l),
-        eq(0l), eq(0l), any(MapOutput.class), eq(true));
+    verify(scheduler).copySucceeded(eq(expectedIdentifier), any(), eq(0L),
+        eq(0L), eq(0L), any(), eq(true));
   }
 
   @Test(timeout = 5000)
