@@ -18,7 +18,6 @@
 
 package org.apache.tez.analyzer.plugins;
 
-import org.apache.tez.common.Preconditions;
 import com.google.common.collect.Lists;
 
 import org.apache.hadoop.conf.Configuration;
@@ -34,6 +33,7 @@ import org.apache.tez.history.parser.datamodel.VertexInfo;
 
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 
 
 /**
@@ -85,14 +85,12 @@ public class SkewAnalyzer extends TezAnalyzerBase implements Analyzer {
 
   private final CSVResult csvResult = new CSVResult(headers);
 
-  private final Configuration config;
-
   private final float minRatio;
   private final float maxRatio;
   private final long maxShuffleBytesPerSource;
 
   public SkewAnalyzer(Configuration config) {
-    this.config = config;
+    super(config);
     maxRatio = config.getFloat(ATTEMPT_SHUFFLE_KEY_GROUP_MAX_RATIO,
         ATTEMPT_SHUFFLE_KEY_GROUP_MAX_RATIO_DEFAULT);
     minRatio = config.getFloat(ATTEMPT_SHUFFLE_KEY_GROUP_MIN_RATIO,
@@ -103,8 +101,7 @@ public class SkewAnalyzer extends TezAnalyzerBase implements Analyzer {
 
   @Override
   public void analyze(DagInfo dagInfo) throws TezException {
-    Preconditions.checkArgument(dagInfo != null, "DAG can't be null");
-    analyzeReducers(dagInfo);
+    analyzeReducers(Objects.requireNonNull(dagInfo, "DAG can't be null"));
   }
 
   private void analyzeReducers(DagInfo dagInfo) {
@@ -214,7 +211,7 @@ public class SkewAnalyzer extends TezAnalyzerBase implements Analyzer {
       if (vertexNumTasks > 1) {
         if (ratio > maxRatio) {
           //input records > 60% of vertex level record count
-          if (inputRecordsCount > (vertexLevelInputRecordsCount * 0.60)) {
+          if (inputRecordsCount > (vertexLevelInputRecordsCount * 0.6f)) {
             List<String> result = Lists.newLinkedList();
             result.add(attemptInfo.getTaskInfo().getVertexInfo().getVertexName());
             result.add(attemptInfo.getTaskAttemptId());
@@ -305,12 +302,7 @@ public class SkewAnalyzer extends TezAnalyzerBase implements Analyzer {
 
   @Override
   public String getDescription() {
-    return "Analyzer reducer skews by mining reducer task counters";
-  }
-
-  @Override
-  public Configuration getConfiguration() {
-    return null;
+    return "Analyze reducer skews by mining reducer task counters";
   }
 
   public static void main(String[] args) throws Exception {
